#pragma once

#include <cassert>

#include <dune/amdis/common/ValueCategory.hpp>

namespace AMDiS {


template <class K, int SIZE>
K sum(Dune::FieldVector<K, SIZE> const& vector)
{
  K result = 0;
  for (int i = 0; i < SIZE; ++i)
    result += vector[i];
  return result;
}
    
    

class OperatorEvaluation
{
  using Scalar = Dune::FieldVector<double, 1>;
    
protected:
  template <class... Args>
  auto evalZotImpl(Args...) const { assert( false ); return 0; }
      
  template <class ValueType>
  auto evalZotImpl(tag::scalar, tag::none, ValueType const& v, 
                    Scalar const& test, Scalar const& trial) const
  {
    return v * test * trial;
  }
  
  template <class... Args>
  auto evalFotImpl(Args...) const { assert( false ); return 0; }
      
  template <size_t I, class ValueType, class Gradient>
  auto evalFotImpl(tag::scalar, VectorComponent<I>, ValueType const& v, 
                    Gradient const& grad_test, Scalar const& trial) const
  {
    return v * grad_test[I] * trial;
  }
      
  template <class ValueType, class Gradient>
  auto evalFotImpl(tag::vector, tag::none, ValueType const& v, 
                    Gradient const& grad_test, Scalar const& trial) const
  {
    return (v * grad_test) * trial;
  }
      
  template <class ValueType, class Gradient>
  auto evalFotImpl(tag::scalar, tag::none, ValueType const& v, 
                    Gradient const& grad_test, Scalar const& trial) const
  {
    return v * sum(grad_test) * trial;
  }
  
  
  template <class... Args>
  auto evalSotImpl(Args...) const { assert( false ); return 0; }
      
  template <size_t R, size_t C, class ValueType, class Gradient>
  auto evalSotImpl(tag::scalar, MatrixComponent<R, C>, ValueType const& v, 
                    Gradient const& grad_test, Gradient const& grad_trial) const
  {
    return v * (grad_test[R] * grad_trial[C]);
  }
      
  template <class ValueType, class Gradient>
  auto evalSotImpl(tag::matrix, tag::none, ValueType const& M, 
                    Gradient const& grad_test, Gradient const& grad_trial) const
  {
    return grad_test * (M * grad_trial);
  }
      
  template <class ValueType, class Gradient>
  auto evalSotImpl(tag::scalar, tag::none, ValueType const& v, 
                    Gradient const& grad_test, Gradient const& grad_trial) const
  {
    return v * (grad_test * grad_trial);
  }
};
  

} // end namespace AMDiS