#pragma once

// std c++ headers
#include <type_traits>

namespace AMDiS
{
  // introduce some shortcuts for integral constants
  // ---------------------------------------------------------------------------
  
  template <int I>
  using int_ = std::integral_constant<int, I>;
  
  /// class that represents a sequence of integers
  template <int...>
  struct Ints {};

  template <bool B>
  using bool_  = std::integral_constant<bool, B>;

  static constexpr bool_<true>  _true  {};
  static constexpr bool_<false> _false {};

  template <int I, int J>
  struct range_
  {
    using type = range_;

    static constexpr int  begin()  { return I; }
    static constexpr int  end()    { return J; }
    static constexpr bool empty()  { return I >= J; }

    constexpr operator bool() const { return I < J; }
  };

  template <int I>
  using empty_range_ = range_<I,I>;


  // some boolean operations
  // ---------------------------------------------------------------------------

  namespace Impl
  {
    template <class... Ts> struct or_;
    
    template <class T0, class... Ts>
    struct or_<T0, Ts...> : bool_<T0::value || or_<Ts...>::value> {};

    template <>
    struct or_<> : std::false_type {};

    template <class... Ts> struct and_;
    
    template <class T0, class... Ts>
    struct and_<T0, Ts...> : bool_<T0::value && and_<Ts...>::value> {};

    template <>
    struct and_<> : std::true_type {};

  } // end namespace Impl

  
  template <class... Ts>
  using and_ = Impl::and_<Ts...>;

  template <bool... Bs>
  using and_c = and_<bool_<Bs>...>;

  template <class... Ts>
  static constexpr bool and_v = and_<Ts...>::value;

  
  template <class... Ts>
  using or_ = Impl::or_<Ts...>;

  template <bool... Bs>
  using or_c = or_<bool_<Bs>...>;

  template <class... Ts>
  static constexpr bool or_v = or_<Ts...>::value;

  
  template <class A>
  using not_ = bool_<!(A::value)>;

  template <class T>
  static constexpr bool not_v = not_<T>::value;


  template <class T, T A, T B>
  using is_equal = std::conditional_t<A == B, std::true_type, std::false_type>;

  template <class T, class... Ts>
  using is_one_of = or_<std::is_same<T, Ts>...>;

} // end namespace AMDiS
