#pragma once

// std c++ headers
#include <algorithm>
#include <cctype>
#include <locale>
#include <string>

namespace AMDiS
{
  // convert all characters in a string to upper case
  inline std::string to_upper(std::string input)
  {
    for (auto & c : input) c = toupper(c);
    return input;
  }
  
  // convert all characters in a string to upper case
  inline std::string to_lower(std::string input)
  {
    for (auto & c : input) c = tolower(c);
    return input;
  }
  
  // trim a string from the left
  inline std::string& ltrim(std::string& str)
  {
    auto it =  std::find_if(str.begin(), str.end(), [](char ch)
    { 
      return !std::isspace<char>(ch, std::locale::classic()); 
    });
    str.erase(str.begin() , it);
    return str;   
  }

  // trim a string from the right
  inline std::string& rtrim(std::string& str)
  {
    auto it =  std::find_if(str.rbegin(), str.rend(), [](char ch)
    { 
      return !std::isspace<char>(ch, std::locale::classic()); 
    });
    str.erase(it.base(), str.end());
    return str;   
  }
  
  // trim a string from both sides
  inline std::string& trim(std::string& str)
  {
    return ltrim(rtrim(str));
  }
  
  // trim a (copy of the) string from both sides
  inline std::string trim_copy(std::string const& str)
  {
    auto s = str;
    return trim(s);
  }
  
  inline std::pair<std::string, std::string> split(std::string const& str, char separator = ':')
  {
    size_t pos = str.find_first_of(separator);
    if (pos == std::string::npos)
      return {"",""};
    
    return {trim_copy(str.substr(0, pos)), trim_copy(str.substr(pos+1))};
  }

} // end namspace AMDiS
