#pragma once

#include <dune/amdis/common/Utility.hpp>
#include <dune/amdis/linear_algebra/SolverInfo.hpp>

namespace AMDiS
{
  /// Interface for Runner / Worker types used in solver classes
  template <class Matrix, class VectorX, class VectorB = VectorX>
  class RunnerInterface
  {
  protected:
    using PreconBase = PreconditionerInterface<Matrix, VectorX, VectorB>;
    
  public:
    /// virtual destructor
    virtual ~RunnerInterface() {}

    /// Is called at the beginning of a solution procedure
    virtual void init(Matrix const& A) = 0;

    /// Is called at the end of a solution procedure
    virtual void exit() = 0;

    /// Solve the system A*x = b
    virtual int solve(Matrix const& A, VectorX& x, VectorB const& b, 
                      SolverInfo& solverInfo) = 0;

    /// Solve the adjoint system A^T*x = b
    virtual int adjointSolve(Matrix const& A, VectorX& x, VectorB const& b, 
                             SolverInfo& solverInfo)
    {
      AMDIS_ERROR_EXIT("Must be implemented by derived class.");
      return 0;
    }
                      
    virtual shared_ptr<PreconBase> getLeftPrecon() { return {}; }
    virtual shared_ptr<PreconBase> getRightPrecon() { return {}; }
  };

} // end namespace AMDiS
