#pragma once

#include <type_traits>

#include <dune/istl/preconditioner.hh>
#include <dune/istl/solvercategory.hh>

#include <dune/amdis/linear_algebra/istl/ISTL_Preconditioner.hpp>

namespace AMDiS {

namespace detail 
{
    template <class T, class = typename T::category >
    std::integral_constant<bool, true> HasCategoryImpl(int);

    template <class T> 
    std::integral_constant<bool, false> HasCategoryImpl(...);

    template <class T>
    using HasCategory = decltype(HasCategoryImpl<T>(int{}));
    
} // end namespace detail
    

template <class Matrix, class VectorX, class VectorB>
class ISTLPreconditioner 
  : public Dune::Preconditioner<VectorX, VectorB>
  , public PreconditionerInterface<Matrix, VectorX, VectorB>
{
public:
    enum {category=Dune::SolverCategory::sequential};
    
    ISTLPreconditioner(std::string preconName, std::string prefix)
      : preconName(preconName)
      , prefix(prefix)
      , preconCreator(prefix)
    {}
    
    /// Implementation of \ref Dune::Preconditioner::pre()
    virtual void pre(VectorX& x, VectorB& b) override { runner.pre(x, b); }

    /// Implementation of \ref Dune::Preconditioner::apply()
    virtual void apply(VectorX& v, const VectorB& d) override { runner.apply(v, d); }

    /// Implementation of \ref Dune::Preconditioner::post()
    virtual void post(VectorX& x) override { runner.post(x); }
    
    // -------------------------------------------------------------------------
    
    /// Implementation of \ref PreconditionerInterface::init()
    virtual void init(Matrix const& A) override
    {
      Matrix& _A = const_cast<Matrix&>(A); // Unschoen!!!
      
      runner = preconCreator.create(preconName, _A);
    };
    
    /// Implementation of \ref PreconditionerInterface::exit()
    virtual void exit() override
    {
      runner.reset();
    };

    /// Implementation of \ref PreconditionerInterface::solve()
    virtual void solve(VectorX const& x, VectorX& y) const override 
    { 
      runner.apply(v, d); 
    }
    
private:
    std::string preconName;
    std::string prefix;
    
    ISTLPreconCreator<Matrix, VectorX, VectorB> preconCreator;
    
    shared_ptr<PreconRunner> runner;
};

} // end namespace AMDiS