#pragma once


#include <boost/numeric/mtl/vector/dense_vector.hpp>

#include <dune/amdis/linear_algebra/LinearAlgebraBase.hpp>

namespace AMDiS
{
  template <class Value>
  using MTLDenseVector = mtl::vec::dense_vector<Value>;
  
  namespace Impl
  {
    template <class Vector>
    class MTLWrapper
      : public BaseWrapper<Vector>
    {
      using Super = BaseWrapper<Vector>;
      
    public:
      using value_type = typename Super::value_type;
      using size_type = typename Super::size_type;
      
      template <class Vector_>
      MTLWrapper(Vector_&& vec) 
        : Super(std::forward<Vector_>(vec)) 
      {}
      
      void resize(size_type s)
      {
        this->vec.change_dim(s);
      }
      
      size_type size() const
      {
        return num_rows(this->vec);
      }
    };
    
  } // end namespace Impl
  
  /// wrap the mtl dense_vector class to provide resize and size member-functions
  template <class Value>
  Impl::MTLWrapper<MTLDenseVector<Value>> wrapper(MTLDenseVector<Value>& vec)
  {
    return {vec};
  }
  
}
