#pragma once

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <dune/functions/functionspacebases/lagrangedgbasis.hh>
#include <dune/functions/functionspacebases/pqknodalbasis.hh>
#include <dune/functions/functionspacebases/pq1nodalbasis.hh>
#include <dune/functions/functionspacebases/taylorhoodbasis.hh>

#include <dune/amdis/common/Mpl.hpp>

namespace AMDiS 
{
  namespace Impl
  {
    // helper class to extract the polynomial degree of a pqk nodal basis
    template <class FeSpace> 
    struct GetDegree : int_<1> {};
    
    template <class GV, int k, class ST>
    struct GetDegree<Dune::Functions::PQkNodalBasis<GV, k, ST> > : int_<k> {};
    
    template <class GV, int k, class ST>
    struct GetDegree<Dune::Functions::LagrangeDGBasis<GV, k, ST> > : int_<k> {};
    
    template <class GV, class ST>
    struct GetDegree<Dune::Functions::PQ1NodalBasis<GV, ST> > : int_<1> {};
    
    template <class GV, class ST>
    struct GetDegree<Dune::Functions::TaylorHoodBasis<GV, ST> > : int_<2> {};
    
  } // end namespace Impl
  
  /// \brief Return the polynomial degree of a given FESpace. Falls back to 1 if unknown.
  template <class FeSpace>
  constexpr int getPolynomialDegree = Impl::GetDegree<FeSpace>::value;
  
} // end namespace AMDiS
