// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:

#include <iostream>

#include <dune/amdis/AMDiS.hpp>
#include <dune/amdis/AdaptInstationary.hpp>
#include <dune/amdis/ProblemInstat.hpp>
#include <dune/amdis/ProblemStat.hpp>
#include <dune/amdis/Terms.hpp>
#include <dune/amdis/common/Literals.hpp>

using namespace AMDiS;

// 1 component with polynomial degree 1
using ElliptParam   = ProblemStatTraits<AMDIS_DIM, AMDIS_DOW, 1>;
using ElliptProblem = ProblemStat<ElliptParam>;
using ElliptProblemInstat = ProblemInstat<ElliptParam>;

int main(int argc, char** argv)
{
  AMDiS::init(argc, argv);
  
  ElliptProblem prob("ellipt");
  prob.initialize(INIT_ALL);
  
  ElliptProblemInstat probInstat("ellipt", prob);
  probInstat.initialize(INIT_UH_OLD);
  
  AdaptInfo adaptInfo("adapt");
  
  using Op = ElliptProblem::OperatorType;
  Op opL, opForce; 
  
  opL.addSOT(1.0);
  prob.addMatrixOperator(opL, 0, 0);
  
  opForce.addZOT( eval([](auto const& x) { return -1.0; }) );
  prob.addVectorOperator(opForce, 0);
  
  
  // set boundary condition
  auto predicate = [](auto const& x){ return x[0] < 1.e-8 || x[1] < 1.e-8; }; // define boundary
  auto dbcValues = [](auto const& x){ return 0.0; }; // set value
  prob.addDirichletBC(predicate, 0, 0, dbcValues);
      
  *prob.getSolution() = 0.0; // maybe not necessary
  
  prob.buildAfterCoarsen(adaptInfo, Flag(0));
  
  // write matrix to file
  {
    mtl::io::matrix_market_ostream out("matrix.mtx");
    out << prob.getSystemMatrix()->getMatrix<0,0>();
  }
  
  prob.solve(adaptInfo);
  prob.writeFiles(adaptInfo, true);
  
  AMDiS::finalize();
  return 0;
}